/*--------------------------------------------------------------------*\
  File:      DevCnnctn.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1998, Matt Bogosian. All rights reserved.
  Description: Header file describing the DevCnnctn class.
  ID:        $Id: DevCnnctn.h,v 1.5 1998/07/01 05:41:24 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include <OS.h>


#ifndef DEV_CNNCTN_H
#define DEV_CNNCTN_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

typedef struct _DevCnnctn *DevCnnctn;


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=- Function Prototypes =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnCreate
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     DevCnnctn * const a_this - a placeholder for the
                     connection to be created.
                 const char a_name[B_FILE_NAME_LENGTH] - the name of
                     the corresponding dev_server addon to load.
                 const port_id a_recv - the port on which messages
                     will be received.
                 const port_id a_send - the port on which messages
                     will be sent (this could be the same as the
                     receiving port in certain cases). Default:
                     B_BAD_PORT_ID.
  Returns:       status_t - the error code associated with the
                     operation (see below).
  Throws:        none
  Description: Class constructor. This attempts to make a connection
      to the dev_server. If construction should fail, the placeholder
      remains untouched. Possible return values are as follows:
      
      B_NO_ERROR - the object was created successfully.
      B_ERROR - the connection could not be made.
      B_NO_MEMORY - there was not enough memory to create the either
          the object or object's Benaphore.
      B_NO_MORE_SEMS - there were no available semaphores to create
          the object's Benaphore.
      B_BAD_PORT_ID - the dev_server accept port could not be found or
          the receiving port was no good.
      B_TIMED_OUT - the dev_server did not respond.
\*--------------------------------------------------------------------*/

status_t DevCnnctnCreate(DevCnnctn * const a_this, const char a_name[B_FILE_NAME_LENGTH], const port_id a_recv, const port_id a_send = B_BAD_PORT_ID);

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnDestroy
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     DevCnnctn * const a_this - the connection to close
                     destroy.
  Returns:       none
  Throws:        none
  Description: Class destructor. This function calls delete_port() on
      both ports handed to the given connection in the constructor.
\*--------------------------------------------------------------------*/

void DevCnnctnDestroy(DevCnnctn * const a_this);

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnLock
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     DevCnnctn a_this - the connection to lock.
  Returns:       status_t - the error code associated with the
                     operation (see below).
  Throws:        none
  Description: Function to exclusively lock access to a given
      connection. If the connection is valid, then this function will
      block until the connection is locked, upon which time the
      function will return B_NO_ERROR. If the connection is not valid,
      the function will return a value less than B_NO_ERROR. A
      successful call to DevCnnctnLock() should always be mirrored by
      exactly one call to DevCnnctnUnlock(). Possible return values
      are as follows:
      
      B_NO_ERROR - success.
      B_BAD_SEM_ID - the connection no longer contains a valid
          Benaphore.
\*--------------------------------------------------------------------*/

status_t DevCnnctnLock(DevCnnctn a_this);

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnRead
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     DevCnnctn a_this - the connection on which to send
                     the message.
                 int32 * const a_msg_code - a placeholder for the
                     message code.
                 void * const a_buf - a placeholder for the message.
                     Default: NULL.
                 const size_t a_buf_size - the size (in bytes) of the
                     message buffer. Default: 0.
                 const uint32 a_flags - the flags. Default: 0.
                 const bigtime_t a_timeout - the timeout
                     (in microseconds). Default: B_INFINITE_TIMEOUT.
  Returns:       ssize_t - the number of bytes read on success, an
                     error code on failure (see below).
  Throws:        none
  Description: Function to read a message from a given connection.
      Possible return values are as follows:
      
      B_BAD_PORT_ID - the connection's sending port is no longer
          valid.
      B_WOULD_BLOCK - a timeout of 0 was specified and the port was
          not available.
      B_TIMED_OUT - the timeout was reached and the message could not
          be read.
\*--------------------------------------------------------------------*/

ssize_t DevCnnctnRead(DevCnnctn a_this, int32 * const a_msg_code, void * const a_buf = NULL, const size_t a_buf_size = 0, const uint32 a_flags = 0, const bigtime_t a_timeout = B_INFINITE_TIMEOUT);

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnReadBufferSize
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     const DevCnnctn a_this - the connection of which to
                     get the buffer size.
                 const uint32 a_flags - the flags. Default: 0.
                 const bigtime_t a_timeout - the timeout
                     (in microseconds). Default: B_INFINITE_TIMEOUT.
  Returns:       ssize_t - the buffer size on success, an error code
                     on failure (see below).
  Throws:        none
  Description: Function to report a given connection's receiving
      port's buffer size. Possible return values are as follows:
      
      B_BAD_PORT_ID - the connection's sending port is no longer
          valid.
      B_WOULD_BLOCK - a timeout of 0 was specified and the port was
          not available.
      B_TIMED_OUT - the timeout was reached and the buffer size could
          not be read.
\*--------------------------------------------------------------------*/

ssize_t DevCnnctnReadBufferSize(const DevCnnctn a_this, const uint32 a_flags = 0, const bigtime_t a_timeout = B_INFINITE_TIMEOUT);

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnSetPortOwner
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     DevCnnctn a_this - the connection of which to change
                     the owner.
                 const thread_id a_team - the team to which to give
                     ownership of the connection.
  Returns:       none
  Throws:        none
  Description: Function to change the ownership of a given
      connection's port(s) to a given team.
\*--------------------------------------------------------------------*/

void DevCnnctnSetPortOwners(DevCnnctn a_this, const thread_id a_team);

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnUnlock
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     DevCnnctn a_this - the connection to unlock.
  Returns:       none
  Throws:        none
  Description: Function to release a given connection. There should be
      exactly one call to DevCnnctnUnlock() for every successful
      call to DevCnnctnLock().
\*--------------------------------------------------------------------*/

void DevCnnctnUnlock(DevCnnctn a_this);

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnWrite
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     DevCnnctn a_this - the connection on which to send
                     the message.
                 const int32 a_msg_code - the message code.
                 const void * const a_msg - the message. Default:
                     NULL.
                 const size_t a_msg_len - the length (in bytes) of the
                     message. Default: 0.
                 const uint32 a_flags - the flags. Default: 0.
                 const bigtime_t a_timeout - the timeout
                     (in microseconds). Default: B_INFINITE_TIMEOUT.
  Returns:       status_t - the error code associated with the
                     operation (see below).
  Throws:        none
  Description: Function to write a message to a given connection.
      Possible return values are as follows:
      
      B_NO_ERROR - success.
      B_BAD_PORT_ID - the connection's sending port is no longer
          valid.
      B_WOULD_BLOCK - a timeout of 0 was specified and the port was
          not available.
      B_TIMED_OUT - the timeout was reached and the message could not
          be sent.
\*--------------------------------------------------------------------*/

status_t DevCnnctnWrite(DevCnnctn a_this, const int32 a_msg_code, const void * const a_msg = NULL, const size_t a_msg_len = 0, const uint32 a_flags = 0, const bigtime_t a_timeout = B_INFINITE_TIMEOUT);

/*--------------------------------------------------------------------*\
  Function name: DevCnnctnWriteBufferSize
  Member of:     public DevCnnctn
  Defined in:    DevCnnctn.cpp
  Arguments:     const DevCnnctn a_this - the connection of which to
                     get the buffer size.
                 const uint32 a_flags - the flags. Default: 0.
                 const bigtime_t a_timeout - the timeout
                     (in microseconds). Default: B_INFINITE_TIMEOUT.
  Returns:       ssize_t - the buffer size on success, an error code
                     on failure (see below).
  Throws:        none
  Description: Function to report a given connection's sending port's
      buffer size. Possible return values are as follows:
      
      B_BAD_PORT_ID - the connection's sending port is no longer
          valid.
      B_WOULD_BLOCK - a timeout of 0 was specified and the port was
          not available.
      B_TIMED_OUT - the timeout was reached and the buffer size could
          not be read.
\*--------------------------------------------------------------------*/

ssize_t DevCnnctnWriteBufferSize(const DevCnnctn a_this, const uint32 a_flags = 0, const bigtime_t a_timeout = B_INFINITE_TIMEOUT);


#endif    // DEV_CNNCTN_H
