/*--------------------------------------------------------------------*\
  File:      PMain.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Header file describing the PMain abstract base class.
  ID:        $Id: PMain.h,v 1.17 1998/07/03 00:25:48 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include <stdio.h>


#ifndef P_MAIN_H
#define P_MAIN_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-= Definitions, Enums, Typedefs, Consts =-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#ifndef DECL_SPEC

#if defined DLL_EXPORT_SYMBOLS
#define DECL_SPEC __declspec(dllexport)
#elif defined DLL_IMPORT_SYMBOLS
#define DECL_SPEC __declspec(dllimport)
#else
#define DECL_SPEC
#endif

#endif    // DECL_SPEC


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-= Structs, Classes =-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#ifdef DLL_EXPORT_SYMBOLS
#pragma export on
#endif

/*--------------------------------------------------------------------*\
  Class name:       PMain
  Inherits from:    none
  New data members: public static const char mk_path_delim - the path
                        delimeter.
                    public static const int mk_exit_code_no_err - the
                        exit code representing a successful execution.
                    public static const int mk_exit_code_err - the
                        exit code representing an unsuccessful
                        execution.
                    public static const int mk_exit_code_bad_args -
                        the exit code representing an error during the
                        parsing of the command line options.
                    public static const int mk_exit_code_no_memory -
                        the exit code representing an error in the
                        allocation of memory.
                    public static const int mk_exit_code_bad_file -
                        the exit code representing an error during
                        file i/o.
                    public static const int mk_exit_code_last - the
                        last exit code defined by this class.
                        Subclasses may define their own exit codes
                        starting at mk_exit_code_last - 1 or
                        mk_exit_code_no_err + 1.
                    protected static const int mk_first_arg_ind - the
                        index of the first real command line argument.
                        This is made available to help subclasses who
                        define the ParseArgs() method.
                    protected static const char mk_opt_esc - the
                        escape character indicating a command line
                        option. This is made available to help
                        subclasses who define the ParseArgs() method.
                    private static PMain *mg_main_instance - the
                        instance of PMain for the process.
                    private static int mg_pid - the process ID.
                    private static FILE *mg_out_file - the file to
                        which the error messages for the process are
                        printed.
                    private static FILE *mg_err_file - the file to
                        which the messages for the process are
                        printed.
                    private static const char *mg_proc_name - the name
                        of the process.
                    private static bool mg_verbose - if set to true,
                        calls to Verbose() will printed, if set to
                        false, they will not.
                    private int &m_argc - the number of command line
                        arguments in m_argv.
                    private const char ** const m_argv - a null-
                        terminated list containing the command line
                        arguments.
                    private bool m_ready_to_run - true if the instance
                        of PMain is ready to run.
                    private bool m_running - true if the instance of
                        PMain is running.
  Description: Abstract base class from which classes may be derived
      to implement handle the main thread of a process. There may only
      be one instance of this class (or one of its derived children)
      in a process. Note: this class is not thread safe. Those
      functions susceptable to bad synchronization have been indicated
      as such in their descriptions. If you intend to call those
      functions from multiple threads, it is STRONGLY recommended that
      you guard access to those functions with real semaphores.
      
      To use, simply derive a class from PMain, define any pure
      virtual hook functions which are required, then instantiate it
      and Start() it:
      
        class MyMain : public PMain
        {
          // Define the required hook functions here
        };
        
        int main(int a_argc, char *a_argv[])
        {
          MyMain *main = new MyMain(a_argc, a_argv);
          
          if (!main->Start())
          {
            MyMain::ErrExit(true, Main::mk_exit_code_err,
              "couldn't run\n");
          }
          
          delete main;
          
          return 0;
        }
      
      Note: as the above example shows, the instance of MyMain must be
      allocated dynamically from the heap using the new operator. Your
      process will barf on exit if it is allocated statically (i.e.,
      from the stack).
\*--------------------------------------------------------------------*/

class DECL_SPEC PMain
{
	public:
	
		// Public static const data members
		static const char mk_path_delim;
		static const int mk_exit_code_no_err;
		static const int mk_exit_code_err;
		static const int mk_exit_code_bad_args;
		static const int mk_exit_code_no_memory;
		static const int mk_exit_code_bad_file;
		static const int mk_exit_code_last;
		
		// Public static member functions
		
/*--------------------------------------------------------------------*\
  Function name: static ArrayLength
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     const void * const * const a_array - a null-
                     terminated array of objects.
  Returns:       size_t - the number of elements in the array.
  Throws:        none
  Description: Function to return the number of elements (not
      including the null terminator) in the array.
\*--------------------------------------------------------------------*/
		
		static size_t ArrayLength(const void * const * const a_array);
		
/*--------------------------------------------------------------------*\
  Function name: static BoundRandom
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     const double a_lo - the lower bound for the random
                     number.
                 const double a_hi - the upper bound for the random
                     number.
  Returns:       double - the random number.
  Throws:        none
  Description: Function to return a pseudo-random number between the
      given bounds (lower bound inclusive, upper bound exclusive).
      This function uses the standard library call random(), so an
      srandom() call should be made to seed the random number
      generator before the first call to this function in any program.
\*--------------------------------------------------------------------*/
		
		static double BoundRandom(const double a_lo, const double a_hi);
		
/*--------------------------------------------------------------------*\
  Function name: static Err
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     const bool a_prnt_proc_name - if this is true, the
                     message will be preceded by the name of the
                     process followed by a colon and a space.
                 const char * const a_fmt - a formatting string a la
                     printf().
                 ... - a variable argument list a la printf().
  Returns:       none
  Throws:        none
  Description: Function to print out an error to the err file.
\*--------------------------------------------------------------------*/
		
		static void Err(const bool a_prnt_proc_name, const char * const a_fmt, ...);
		
/*--------------------------------------------------------------------*\
  Function name: static ErrDump
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     const bool a_prnt_proc_name - if this is true, the
                     message will be preceded by the name of the
                     process followed by a colon and a space.
                 const char * const a_fmt - a formatting string a la
                     printf().
                 ... - a variable argument list a la printf().
  Returns:       none
  Throws:        none
  Description: Function to print out a fatal error to the err file and
      dump core.
\*--------------------------------------------------------------------*/
		
		static void ErrDump(const bool a_prnt_proc_name, const char * const a_fmt, ...);
		
/*--------------------------------------------------------------------*\
  Function name: static ErrExit
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     const bool a_prnt_proc_name - if this is true, the
                     message will be preceded by the name of the
                     process followed by a colon and a space.
                 const int a_err - the exit code.
                 const char * const a_fmt - a formatting string a la
                     printf().
                 ... - a variable argument list a la printf().
  Returns:       none
  Throws:        none
  Description: Function to print out a fatal error to the err file and
      exit the process. This function calls the Exit() member function
      and is bound by all caveats applicable to that function.
\*--------------------------------------------------------------------*/
		
		static void ErrExit(const bool a_prnt_proc_name, const int a_err, const char * const a_fmt, ...);
		
/*--------------------------------------------------------------------*\
  Function name: static ErrFile
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     FILE *a_err_file - the new err file to which error
                     messages will be printed. Note: if this is null,
                     the err file remains unchanged. Default: NULL.
  Returns:       FILE * - the old err file.
  Throws:        none
  Description: Function to set the err file to which error messages
      will be printed.
\*--------------------------------------------------------------------*/
		
		static FILE *ErrFile(FILE *a_err_file = NULL);
		
/*--------------------------------------------------------------------*\
  Function name: static Exit
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     const int a_err - the exit code.
  Returns:       none
  Throws:        none
  Description: Function to exit the process. This function suffers
      from the same synchronization problems as the PMain constructor
      and the PMain Start() method, which is that if two calls are
      made to this function at the same time, neither will succeed. If
      your process uses multiple threads, it is STRONGLY recommended
      that you guard calls to this function with a real semaphore to
      insure correct behavior.
\*--------------------------------------------------------------------*/
		
		static void Exit(const int a_err);
		
/*--------------------------------------------------------------------*\
  Function name: static ExitNoMem
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Function to print out a fatal out-of-memory error to
      the err file and exit the process with an exit code of
      mk_exit_code_no_memory. This function calls the Exit() member
      function and is bound by all caveats applicable to that
      function.
\*--------------------------------------------------------------------*/
		
		static void ExitNoMem(void);
		
/*--------------------------------------------------------------------*\
  Function name: inline static GetMainInstance
  Member of:     public PMain
  Arguments:     none
  Returns:       PMain * - the main instance of the PMain class on
                     success, null on failure (i.e., there is no
                     instance).
  Throws:        none
  Description: Function to get the main instance of the PMain class.
\*--------------------------------------------------------------------*/
		
		inline static PMain *GetMainInstance(void)
		{
			return mg_main_instance;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline static GetPID
  Member of:     public PMain
  Arguments:     none
  Returns:       int - the process ID.
  Throws:        none
  Description: Function to get the process ID.
\*--------------------------------------------------------------------*/
		
		inline static int GetPID(void)
		{
			return mg_pid;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline static IsVerbose
  Member of:     public PMain
  Arguments:     none
  Returns:       bool - true if verbose is set to on, false if not.
  Throws:        none
  Description: Function to get the verbosity of the process.
\*--------------------------------------------------------------------*/
		
		inline static bool IsVerbose(void)
		{
			return mg_verbose;
		}
		
/*--------------------------------------------------------------------*\
  Function name: static Out
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     const bool a_prnt_proc_name - if this is true, the
                     message will be preceded by the name of the
                     process followed by a colon and a space.
                 const char * const a_fmt - a formatting string a la
                     printf().
                 ... - a variable argument list a la printf().
  Returns:       none
  Throws:        none
  Description: Function to print out a message to the out file.
\*--------------------------------------------------------------------*/
		
		static void Out(const bool a_prnt_proc_name, const char * const a_fmt, ...);
		
/*--------------------------------------------------------------------*\
  Function name: static OutFile
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     FILE *a_out_file - the new out file to which messages
                     will be printed. Note: if this is NULL, the out
                     file remains unchanged. Default: NULL.
  Returns:       FILE * - the old out file.
  Throws:        none
  Description: Function to set the out file to which messages will be
      printed.
\*--------------------------------------------------------------------*/
		
		static FILE *OutFile(FILE *a_out_file = NULL);
		
/*--------------------------------------------------------------------*\
  Function name: inline static ProcName
  Member of:     public PMain
  Arguments:     none
  Returns:       const char * const - the process name.
  Throws:        none
  Description: Function to return the name of the process as indicated
      by the zero-index command line argument.
\*--------------------------------------------------------------------*/
		
		inline static const char * const ProcName(void)
		{
			return mg_proc_name;
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline static SetVerbose
  Member of:     public PMain
  Arguments:     const bool a_verbose - true to set verbose to on,
                     false to set it to off.
  Returns:       none
  Throws:        none
  Description: Function to set the verbosity of the process.
\*--------------------------------------------------------------------*/
		
		inline static void SetVerbose(const bool a_verbose)
		{
			mg_verbose = a_verbose;
		}
		
/*--------------------------------------------------------------------*\
  Function name: static Verbose
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     const bool a_prnt_proc_name - if this is true, the
                     message will be preceded by the name of the
                     process followed by a colon and a space.
                 const char * const a_fmt - a formatting string a la
                     printf().
                 ... - a variable argument list a la printf().
  Returns:       none
  Throws:        none
  Description: Function to print out a message to the out file if and
      only if IsVerbose() resolves to true. Note: verbosity may be set
      with the SetVerbose() method.
\*--------------------------------------------------------------------*/
		
		static void Verbose(const bool a_prnt_proc_name, const char * const a_fmt, ...);
		
		// Public member functions
		
/*--------------------------------------------------------------------*\
  Function name: PMain
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
  Throws:        none
  Description: Class constructor. This will only create a runnable
      class one time. However, the runnable instance may not
      necessarily result from the first instantion if two
      instantiations are attempted at the same time. To avoid the
      this situation, either make sure you only make one call to this
      function, or wait to create other threads in the process until
      after the first PMain object is instantiated. A good place to do
      this is in the Run() method. Or, if your process uses multiple
      threads and must call this function from within those threads,
      it is STRONGLY recommended that you guard calls to this function
      with real semaphores to insure correct behavior.
\*--------------------------------------------------------------------*/
		
		PMain(int &a_argc, const char ** const a_argv);
		
/*--------------------------------------------------------------------*\
  Function name: virtual ~PMain
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     none
  Throws:        none
  Description: Class destructor.
\*--------------------------------------------------------------------*/
		
		virtual ~PMain(void);
		
/*--------------------------------------------------------------------*\
  Function name: inline IsReadyToRun const
  Member of:     public PMain
  Arguments:     none
  Returns:       bool - true if the object is ready to run, false if
                     not.
  Throws:        none
  Description: Function to check to see if the object is ready to run.
      This function will return false if the object is currently
      running or if the construction failed.
\*--------------------------------------------------------------------*/
		
		inline bool IsReadyToRun(void) const
		{
			return (m_ready_to_run
				&& !m_running);
		}
		
/*--------------------------------------------------------------------*\
  Function name: inline IsRunning const
  Member of:     public PMain
  Arguments:     none
  Returns:       bool - true if the object is running, false if not.
  Throws:        none
  Description: Function to check to see if the object is running.
\*--------------------------------------------------------------------*/
		
		inline bool IsRunning(void) const
		{
			return m_running;
		}
		
/*--------------------------------------------------------------------*\
  Function name: virtual ParseOpts
  Member of:     public PMain
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
  Returns:       int - the error code resulting from the parsing of
                     the arguments (mk_exit_code_no_err if there was
                     no error).
  Throws:        none
  Description: Hook function called by the Start() method to parse the
      command-line function. Note: this function should never be
      called explicitly; always use the Start() method to set the
      object in motion.
\*--------------------------------------------------------------------*/
		
		virtual int ParseOpts(int &/*a_argc*/, const char ** const /*a_argv*/)
		{
			return mk_exit_code_no_err;
		}
		
/*--------------------------------------------------------------------*\
  Function name: Start
  Member of:     public PMain
  Defined in:    PMain.cpp
  Arguments:     none
  Returns:       bool - true if the call succeeded, false if not.
  Throws:        none
  Description: Function to set the object in motion. This method makes
      a few checks, then calls the Run() derived hook function. This
      method will only work once, and in rare cases, it will fail the
      first time if two calls to this method are made at the same
      time. Again, to avoid this situation, wait to create other
      threads in the process until after this method is called in the
      main thread. Or make sure you only call this function once.
      Again, if your process uses multiple threads and must call this
      function from within separate threads, it is STRONGLY
      recommended that you guard calls to this function with real
      semaphores to insure correct behavior.
\*--------------------------------------------------------------------*/
		
		bool Start(void);
	
	protected:
	
		// Protected static const data members
		static const int mk_first_arg_ind;
		static const char mk_opt_esc;
		
		// Protected static member functions
		
/*--------------------------------------------------------------------*\
  Function name: static ShiftArgs
  Member of:     protected PMain
  Defined in:    PMain.cpp
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
                 const int a_which - the argument to remove.
  Returns:       const char * - the argument which was removed (null
                     if none).
  Throws:        none
  Description: Function to remove a command line argument from a given
      null-terminated array. This is provided to use with the
      ParseOpts() function.
\*--------------------------------------------------------------------*/
		
		static const char *ShiftArgs(int &a_argc, const char ** const a_argv, int a_which);
		
		// Protected member functions
		
/*--------------------------------------------------------------------*\
  Function name: virtual Help const
  Member of:     protected PMain
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Hook function to called to print the help information
      for the program.
\*--------------------------------------------------------------------*/
		
		virtual void Help(void) const
		{
			Usage();
		}
		
/*--------------------------------------------------------------------*\
  Function name: virtual Run
  Member of:     protected PMain
  Arguments:     int &a_argc - the number of command line arguments.
                 const char ** const a_argv - the command line
                     arguments.
  Returns:       none
  Throws:        none
  Description: Hook function called by the Start() method. This
      function runs the main thread of the process. Note: this
      function should never be called explicitly; always use the
      Start() method to set the object in motion.
\*--------------------------------------------------------------------*/
		
		virtual void Run(int &a_argc, const char ** const a_argv) = NULL;
		
/*--------------------------------------------------------------------*\
  Function name: virtual Usage const
  Member of:     protected PMain
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Hook function to called to print the usage information
      for the program.
\*--------------------------------------------------------------------*/
		
		virtual void Usage(void) const = NULL;
		
/*--------------------------------------------------------------------*\
  Function name: virtual Version const
  Member of:     protected PMain
  Arguments:     none
  Returns:       none
  Throws:        none
  Description: Hook function to called to print the version
      information for the program.
\*--------------------------------------------------------------------*/
		
		virtual void Version(void) const = NULL;
	
	private:
	
		// Private static data members
		static PMain *mg_main_instance;
		static int mg_pid;
		static FILE *mg_out_file;
		static FILE *mg_err_file;
		static const char *mg_proc_name;
		static bool mg_verbose;
		
		// Private data members
		int &m_argc;
		const char ** const m_argv;
		bool m_ready_to_run;
		bool m_running;
		
		// Private static member functions
		
/*--------------------------------------------------------------------*\
  Function name: static Print
  Member of:     private PMain
  Defined in:    PMain.cpp
  Arguments:     FILE *a_file - the file to which to print the
                     message.
                 const bool a_prnt_proc_name - if this is true, the
                     message will be preceded by the name of the
                     process followed by a colon and a space.
                 const char * const a_fmt - a formatting string a la
                     printf().
                 const va_list a_args - a variable argument list a la
                     printf().
  Returns:       none
  Throws:        none
  Description: Function to print out a message to the given file.
\*--------------------------------------------------------------------*/
		
		static void Print(FILE *a_file, const bool a_prnt_proc_name, const char * const a_fmt, const va_list a_args);
		
		// Private prohibitted member functions
		PMain(const PMain &a_obj);
		PMain &operator=(const PMain &a_obj);
};

#ifdef DLL_EXPORT_SYMBOLS
#pragma export off
#endif


#endif    // P_MAIN_H
