/*--------------------------------------------------------------------*\
  File:      ThruputAddOn.cpp
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1998, Matt Bogosian. All rights reserved.
  Description: Source file containing the member functions for the
      ThruputAddOn class.
  ID:        $Id: ThruputAddOn.cpp,v 1.1 1998/07/03 12:51:26 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include "ThruputAddOn.h"
#include "ThruputWindow.h"


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-= Function Definitions =-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#pragma export on

//====================================================================
extern "C" __declspec(dllexport) status_t dvsrvrNewAddon(const void * const a_data, AddOn ** const a_add_on)
//====================================================================
{
	status_t err(B_NO_MEMORY);
	
	// Be a good boy and try create the add-on with the data given
	// to us
	AddOn *add_on;
	
	// Check for an allocation error
	if ((add_on = new ThruputAddOn(a_data, &err)) == NULL
		|| err != B_NO_ERROR)
	{
		delete add_on;
		return err;
	}
	
	// We're all set to rock and roll
	*a_add_on = add_on;
	
	return B_NO_ERROR;
}

#pragma export off

//====================================================================
ThruputAddOn::ThruputAddOn(const void * const a_data, status_t  * const a_err) :
//====================================================================
	ThruputAddOn_SUPER(a_data, a_err)
{
	;
}

//====================================================================
ThruputAddOn::~ThruputAddOn(void)
//====================================================================
{
	;
}

//====================================================================
status_t ThruputAddOn::Start(const port_id a_send, const port_id /*a_recv*/)
//====================================================================
{
	status_t err(B_NO_MEMORY);
	
	// Try to create the window and add it to the add-on as a
	// looper
	ThruputWindow *win;
	
	// Check for an allocation error
	if ((win = new ThruputWindow(a_send, this, &err)) == NULL
		|| err != B_NO_ERROR)
	{
		delete win;
		return err;
	}
	
	// We're all set to rock and roll
	win->Show();
	
	return B_NO_ERROR;
}
