/*--------------------------------------------------------------------*\
  File:      ThruputWindow.h
  Creator:   Matt Bogosian <mbogosian@usa.net>
  Copyright: (c)1997, 1998, Matt Bogosian. All rights reserved.
  Description: Header file describing the ThruputWindow class.
  ID:        $Id: ThruputWindow.h,v 1.3 1998/07/01 21:41:50 mattb Exp $
  Conventions:
      #defines - all uppercase letters with words separated by
          underscores.
          (E.G., #define MY_DEFINE 16).
      New data types (classes, structs, typedefs, etc.) - begin with
          an uppercase letter followed by lowercase words separated by
          uppercase letters.
          (E.G., typedef int MyTypedef;).
      Global constants (declared with const) - begin with "k_"
          followed by lowercase words separated by underscores.
          (E.G., const int k_my_constant(16);).
      Global variables - begin with "g_" followed by lowercase words
          separated by underscores.
          (E.G., int g_my_global;).
      Local variables - begin with a lowercase letter followed by
          lowercase words separated by underscores.
          (E.G., int my_local;).
      Argument variables - begin with "a_" followed by lowercase words
          separated by underscores.
          (E.G., ..., int a_my_argument, ...).
      Functions - begin with a lowercase letter followed by lowercase
          words separated by uppercase letters.
          (E.G., void myFunction(void);).
      Static member constants (declared with const) - begin with "mk_"
          followed by lowercase words separated by underscores.
          (E.G., static const int MyClass::mk_my_constant(16);).
      Static member variables - begin with "mg_" followed by lowercase
          words separated by underscores.
          (E.G., static int MyClass::mg_my_global(16);).
      Member variables - begin with "m_" followed by lowercase words
          separated by underscores.
          (E.G., int m_my_member;).
      Member Functions - begin with an uppercase letter followed by
          lowercase words separated by uppercase letters.
          (E.G., void MyClass::MyFunction(void);).
\*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-=- Included Files -=-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

#include <Window.h>


#ifndef THRUPUT_WINDOW_H
#define THRUPUT_WINDOW_H


/*--------------------------------------------------------------------*\
  =-=-=-=-=-=-=-=-=-=-=-=-= Structs, Classes =-=-=-=-=-=-=-=-=-=-=-=-=
\*--------------------------------------------------------------------*/

class AddOn;
class ThruputView;

/*--------------------------------------------------------------------*\
  Class name:       ThruputWindow
  Inherits from:    public BWindow
  New data members: private status_t m_status - the status of the
                        object.
                    private AddOn *m_add_on - the window's dev_server
                        add-on interface.
                    private ThruputView *m_main_view - the window's
                        main view.
  Description: Class to implement a window in which an approximation
      of the instantaneous thruput to a selected device is constantly
      displayed.
\*--------------------------------------------------------------------*/

#ifndef ThruputWindow_SUPER
#define ThruputWindow_SUPER BWindow
#endif

class ThruputWindow : public ThruputWindow_SUPER
{
	public:
	
		// Public member functions
		
/*--------------------------------------------------------------------*\
  Function name: ThruputWindow
  Member of:     public ThruputWindow
  Defined in:    ThruputWindow.cpp
  Arguments:     const port_id a_port - the thruput message port (this
                     is passed straight to the ThruputView).
                 AddOn * const a_add_on - the dev_server add-on
                     interface.
                 status_t * const a_err - a placeholder for the error
                     encountered during construction (see below).
                     Default: NULL.
  Throws:        none
  Description: Class constructor. You must check the value of a_err or
      the return value of Status() after invoking the constructor to
      see if the object is usable. See the Status() function (below)
      for possible error values.
\*--------------------------------------------------------------------*/
		
		ThruputWindow(const port_id a_port, AddOn * const a_add_on, status_t * const a_err = NULL);
		
/*--------------------------------------------------------------------*\
  Function name: virtual ~ThruputWindow
  Member of:     public ThruputWindow
  Defined in:    ThruputWindow.cpp
  Arguments:     none
  Throws:        none
  Description: Class destructor.
\*--------------------------------------------------------------------*/
		
		virtual ~ThruputWindow(void);
		
/*--------------------------------------------------------------------*\
  Function name: virtual QuitRequested
  Member of:     public ThruputWindow
  Defined in:    ThruputWindow.cpp
  Arguments:     none
  Returns:       bool - true if the window will quit, false otherwise.
  Throws:        none
  Description: Derived hook function.
\*--------------------------------------------------------------------*/
		
		virtual bool QuitRequested(void);
		
/*--------------------------------------------------------------------*\
  Function name: inline Status const
  Member of:     public ThruputWindow
  Arguments:     none
  Returns:       status_t - B_NO_ERROR if the object is valid, another
                     error code if not (see below).
  Throws:        none
  Description: Function to check if the object is valid. This function
      must be called after creating the object to insure that its
      instantiation was successful. Possible return values are as
      follows:
      
      B_NO_ERROR - the object was created successfully.
      B_NO_MEMORY - on construction, there was not enough memory
          available to create the object.
\*--------------------------------------------------------------------*/
		
		inline int Status(void) const
		{
			return m_status;
		}
	
	private:
	
		// Private data members
		status_t m_status;
		AddOn *m_add_on;
		ThruputView *m_main_view;
};


#endif    // THRUPUT_WINDOW_H
